@@ -49,7 +49,10 @@ typedef enum chantype {
     cTestPoint,
 
     /**  FrStat calibration data */
-    cStatic
+    cStatic,
+
+    /**  FrSimData simulated data */
+    cSimData
 
 } chantype_t;
 
