@@ -12,12 +12,6 @@ extern "C"
 {
 #endif /* __cplusplus */
 
-#if WIN32 || WIN64
-#define DLL_EXPORT __declspec(dllexport)
-#else
-#define DLL_EXPORT
-#endif /* WIN32 || WIN64 */
-
 #include "channel.h"
 
 typedef          int  int4_type;
@@ -34,7 +28,11 @@ struct signal_conv1;
 
 struct daq_private_;
 
-/**  \mainpage NDS2 Application Program Interface
+/** \if (DEVELOPER_SECTION)
+  * \page NDS2_C_API NDS2 Application Program Interface
+  * \else
+  * \mainpage NDS2 Application Program Interface
+  * \endif
   *  This module defines the client interface to both versions of the Network
   *  Data Server (classic NDS and NDS2). It is based on the "daq_" function 
   *  set originally defined by Alex Ivanov for the classic NDS. The C-functions
@@ -449,6 +447,17 @@ daq_get_data_length(daq_t *daq, const char* channel);
 DLL_EXPORT chan_req_t*
 daq_get_channel_status(daq_t *daq, const char* channel);
 
+/**  Last error message from the server.
+  *  \brief Get the last message
+  *  \param daq Pointer to client status structure.
+  *  \param buf Buffer to receive null terminated message string.
+  *  \param max_len length of buf.
+  *  \param len pointer to int to receive length of message.
+  *  \return Number of message bytes or -1.
+  */
+DLL_EXPORT int
+daq_get_last_message(daq_t *daq, char* buf, size_t max_len, int* len);
+
 /**  Copy the data from a specified channel name to the target address.
   *  The data are converted to floats and scaled according to the linear
   *  transform specified by the NDS server. Complex data are not copied
