@@ -147,7 +147,7 @@ listener (void* a) {
                            (socklen_t) sizeof(*(lstn->listener_addr)));
                 if (rc < 0) {
                     if (errno != EADDRINUSE && errno != 0) {
-                        fprintf (stderr, "bind(); errno=%d\n", errno);
+                        nds_logging_print_errno ( "bind()" );
                         return NULL;
                     }
                 } else {
@@ -155,7 +155,7 @@ listener (void* a) {
                 }
             }
         } else {
-            fprintf (stderr, "bind(); errno=%d\n", errno);
+            nds_logging_print_errno ( "bind()" );
             return NULL;
         }
     }
@@ -207,7 +207,7 @@ listener (void* a) {
 #endif
 
     if (listen (listenfd, 2) < 0) {
-        fprintf (stderr, "listen(); errno=%d\n", errno);
+        nds_logging_print_errno ( "listen()" );
         return NULL;
     }
     
@@ -232,7 +232,7 @@ listener (void* a) {
             if (errno == ECONNABORTED) {
                 continue;
             } else {
-                fprintf (stderr, "accept(); errno=%d\n", errno);
+                nds_logging_print_errno( "accept()" );
                 return NULL;
             }
         }
@@ -248,17 +248,29 @@ listener (void* a) {
                              (const char *) &rcvbuf_size, 
                              sizeof (rcvbuf_size));
             if (rc} {
-                fprintf (stderr, "setsockopt(%d, %d); errno=%d\n", 
-                         connfd, rcvbuf_size, errno);
+                char lead[2048];
+
+                sprintf( lead,
+                         "setsockopt(%d, %d)"
+                         connfd, rcvbuf_size );
+                nds_logging_print_errno( lead ); 
             }
 #endif
 
             rcvbuf_size = -1;
             rc = getsockopt (connfd, SOL_SOCKET, SO_RCVBUF, 
                              (const char *) &rcvbuf_size, &rcvbuf_size_len);
-            if (rc) fprintf (stderr, "getsockopt(%d, %d); errno=%d\n", 
-                             connfd, rcvbuf_size, errno);
-            else printf ("connfd RCVBUF size is %d\n", rcvbuf_size);
+            if (rc) {
+                char lead[2048];
+
+                sprintf( lead,
+                         "getsockopt(%d, %d)"
+                         connfd, rcvbuf_size );
+                nds_logging_print_errno( lead ); 
+            }
+            else {
+                nds_logging_printf( "connfd RCVBUF size is %d\n", rcvbuf_size );
+            }
         }
 #endif
 
@@ -266,7 +278,7 @@ listener (void* a) {
         {
             int err_no;
 #ifdef not_def
-            fprintf (stderr, "#connfd=%d\n", connfd);
+            nds_logging_printf (stderr, "#connfd=%d\n", connfd);
 #endif
             daq -> conceal -> datafd = connfd;
 
@@ -293,13 +305,12 @@ listener (void* a) {
                                      (void *) lstn);
 #endif /* _WIN32 */
             if (err_no) {
-                fprintf (stderr, "failed to spawn interpreter thread; err=%d", 
-                         err_no);
+                nds_logging_print_errno( "failed to spawn interpreter thread" );
                 close (connfd);
             } else {
 #ifdef not_def
-                fprintf (stderr, "#interpreter started; tid=%d\n", 
-                         lstn -> interpreter_tid);
+                nds_logging_printf( "#interpreter started; tid=%d\n", 
+                                    lstn -> interpreter_tid );
 #endif
 #if _WIN32
 #else /* _WIN32 */
@@ -383,9 +394,7 @@ daq_initialize (daq_listen_t* lstn, daq_t* daq, int* tcp_port,
     err_no = pthread_create (&lstn -> listener_tid, NULL, (void *(*)(void *))listener, (void *) lstn);
 #endif /* _WIN32 */
     if (err_no) {
-        fprintf (stderr, 
-                 "pthread_create() failed to spawn a listener thread; err=%d", 
-                 err_no);
+        nds_logging_print_errno( "pthread_create() failed to spawn a listener thread" );
 #if _WIN32
         ReleaseMutex (&lstn -> lock);
         CloseHandle (&lstn -> lock);
