@@ -41,6 +41,9 @@ extern "C" {
     /** Data connection socket 
      */
     nds_socket_type datafd;
+
+    size_t max_command_count;
+    size_t cur_command_count;
   };
 
   typedef struct daq_private_ daq_private_t;
