@@ -4,6 +4,18 @@
 #ifndef DAQC_NDS1_H
 #define DAQC_NDS1_H
 
+/*
+ * Cycle the connection to the nds1 server.  This will reuse the
+ * host and port information.  This only works after a successful
+ * connection.  It is a work around for ticket 282 where the nds1
+ * server sends DAQD_ERROR after some number of requests.
+ *
+ * This function is used to help transparently cycle the connection
+ * and reset whatever is causing nds1 issues.
+ */
+int
+nds1_reconnect (daq_t *daq);
+
 /*
  *  Connect to the DAQD server on the host identified by `ip' address.
  *  Returns zero if OK or the error code if failed.
@@ -19,6 +31,13 @@ nds1_connect (daq_t *daq, const char* host, int port);
 int
 nds1_disconnect (daq_t *daq);
 
+/*
+ *  Get the most recent error message recorded by the server.
+ *  Returns zero if OK or the error code if failed.
+ */
+int
+nds1_get_last_message (daq_t *daq, char* buf, size_t max_len, int* len);
+
 /*
  *  Initialize the nds1 client
  *  Returns zero if OK or the error code if failed.
