@@ -28,6 +28,7 @@
 #define MAX_GROUP_LIST   4096
 #define MAX_EPOCH_LIST   65536UL
 #define MAX_CHANNEL_LIST 209715200
+#define MAX_MESSAGE_LEN  128UL
 
 #define NEW_VECT(type,dim) ((type*)malloc(dim*sizeof(type)))
 
@@ -94,6 +95,24 @@ dump_data(daq_t* daq, size_t spl, const char* fmt) {
 	    }
 	    break;
 	}
+
+	case _32bit_uint: {
+	    uint32_t* p =(uint32_t*)NULL;
+            const char* ifmt = "%8u";
+            if (fmt && *fmt) ifmt = fmt;
+	    p = (uint32_t*)(daq_get_block_data( daq )
+			   + chan->offset);
+	    for (j=0; j<N; j+=spl) {
+		printf("%5" PRISIZE_T, j);
+		for (k=j; k<j+spl && k<N; ++k) {
+		    printf(" ");
+		    printf(ifmt, (unsigned int)*p++);
+		}
+		printf("\n");
+	    }
+	    break;
+	}
+
 	case _32bit_integer: {
 	    int32_t* p =(int32_t*)NULL;
             const char* ifmt = "%8i";
@@ -186,6 +205,7 @@ main(int argc, const char* argv[]) {
     enum nds_version vrsn = nds_try;
     enum pgm_mode mode   = chan_data;
     enum chantype c_type = cUnknown;
+    int daq_test_debug = 0;
 
     int retval = 0;
 
@@ -203,20 +223,24 @@ main(int argc, const char* argv[]) {
     int iarg;
     int syntax=0;
 
-#if 0
-    /*
-     * This is an example of how to enable debugging.
-     */
-    nds_logging_enable( NDS_LOG_GROUP_CONNECTION );
-    nds_logging_enable( NDS_LOG_GROUP_VERBOSE_ERRORS );
-    nds_logging_enable( NDS_LOG_GROUP_TRACE_ENTRY_EXIT );
-    nds_logging_enable( NDS_LOG_GROUP_STATUS_UPDATE );
-    nds_logging_enable( NDS_LOG_GROUP_USER );
-#endif /* 0 */
-
+    /*-----------------------------------------------------
+     *   Initialize error logging
+     *-----------------------------------------------------*/
 
     nds_logging_init( );
 
+    /*
+     *   Initialize nds client library
+     */
+    rc = daq_startup();
+    if (rc) {
+        fprintf(stderr, "Failure in daq_startup. error code: %d.\n", rc);
+        return 1;
+    }
+
+    /*
+     *   Parse the command line arguments
+     */
     for (iarg=1; iarg<argc && *(argv[iarg]) == '-'; ++iarg) {
 	if (!strcmp(argv[iarg], "-c")) {
 	    mode = cmd_exec;
@@ -227,6 +251,12 @@ main(int argc, const char* argv[]) {
 	    else                delta = strtol(argv[iarg], 0, 0);
 	}
 
+	/*------------------------------  End debug level             */
+	else if (!strcmp(argv[iarg], "--debug")) {
+	    if (++iarg == argc) daq_test_debug = 1;
+	    else                daq_test_debug = strtol(argv[iarg], 0, 0);
+	} 
+
 	/*------------------------------  End gps time               */
 	else if (!strcmp(argv[iarg], "-e")) {
 	    if (++iarg == argc) syntax = 1;
@@ -304,9 +334,21 @@ main(int argc, const char* argv[]) {
 	    break;
 	}
     }
+
+    /* optionally turn on debugging 
+     * Note that this probably needs to be smartened by allowing user to 
+     * specify a group mask and/or printout levels.
+     */
+    if (daq_test_debug) { 
+        nds_logging_enable( NDS_LOG_GROUP_CONNECTION );
+        nds_logging_enable( NDS_LOG_GROUP_VERBOSE_ERRORS );
+        nds_logging_enable( NDS_LOG_GROUP_TRACE_ENTRY_EXIT );
+        nds_logging_enable( NDS_LOG_GROUP_STATUS_UPDATE );
+        nds_logging_enable( NDS_LOG_GROUP_USER );
+    }
+
     if ( mode == server_protocol ) {
 	if (!port_id) port_id = DAQD_PORT;
-	daq_startup();
 	rc = daq_connect(&daq, node_id, port_id, vrsn);
 	if ( nds_logging_check( NDS_LOG_GROUP_USER, 10 ) ) {
 	    nds_logging_printf( "DEBUG: %06d - %s - rc %d\n",
@@ -325,7 +367,6 @@ main(int argc, const char* argv[]) {
 	exit ( 0 );
     } else if ( mode == server_version ) {
 	if (!port_id) port_id = DAQD_PORT;
-	daq_startup();
 	rc = daq_connect(&daq, node_id, port_id, vrsn);
 	if ( nds_logging_check( NDS_LOG_GROUP_USER, 10 ) ) {
 	    nds_logging_printf( "DEBUG: %06d - %s - rc %d\n",
@@ -404,7 +445,6 @@ main(int argc, const char* argv[]) {
 			    __LINE__,
 			    __FILE__ );
     }
-    daq_startup();
 
     if ( nds_logging_check( NDS_LOG_GROUP_USER, 10 ) ) {
 	nds_logging_printf( "DEBUG: %06d - %s\n",
@@ -761,9 +801,14 @@ main(int argc, const char* argv[]) {
                 printf("Warning: Can not check data availability: %s\n", 
                        daq_strerror(rc));
             } else if (rc != DAQD_OK) {
+                char msg[MAX_MESSAGE_LEN];
+                int msglen;
                 printf("Data not available - request ignored: %s\n", 
                        daq_strerror(rc));
                 free(channel_list);
+                if (!daq_get_last_message(&daq, msg, MAX_MESSAGE_LEN, &msglen)){
+                    printf("last server error: %s\n", msg);
+                }
                 daq_disconnect(&daq);
                 daq_recv_shutdown(&daq);
                 return rc;
@@ -772,7 +817,12 @@ main(int argc, const char* argv[]) {
             /*   Now get the data data */
 	    rc = daq_request_data(&daq, gps_start, gps_end, delta);
 	    if (rc) {
+                char msg[MAX_MESSAGE_LEN];
+                int msglen;
 		printf("Error in daq_request_data: %s\n", daq_strerror(rc));
+                if (!daq_get_last_message(&daq, msg, MAX_MESSAGE_LEN, &msglen)){
+                    printf("last server error: %s\n", msg);
+                }
                 retval = NDS_EXIT_STATUS_FAIL;
 	    } else {
 		time_t gps, dt=delta;
