@@ -276,6 +276,22 @@ daq_destroy(daq_t *daq) {
     daq -> num_chan_alloc   = 0;
 }
 
+static
+int daq_cycle_conn_if_needed(daq_t *daq)
+{
+
+    int non_blocking = 0;
+
+    if (!daq || daq->nds_versn != nds_v1)
+        return DAQD_OK;
+    daq_private_t *Private = daq->conceal;
+    /* The three is not completely arbitrary, the full handshake is 2 commands. */
+    if (Private->cur_command_count < Private->max_command_count ||
+        Private->max_command_count < 3)
+        return DAQD_OK;
+    return nds1_reconnect(daq);
+}
+
 /*  daq_send (daq_t *daq, char *command)
  *  Send `command' to DAQD, read and return response code
  */
@@ -300,10 +316,14 @@ typedef size_t write_len_t;
         nds_logging_printf( "INFO: %s: entry\n",
                             METHOD );
     }
+    resp = daq_cycle_conn_if_needed(daq);
+    if (resp != DAQD_OK)
+        return resp;
     cmdl = strlen( command );
     if (!cmdl) return DAQD_OK;
     if (command[cmdl-1] == '\n') {
-	buf=command;
+        // Explicitly remove const type specifier to suppress warning
+	buf=(char*)command;
     } else {
 	buf = (char*)malloc(cmdl+1);
 	strcpy(buf, command);
@@ -319,7 +339,7 @@ typedef size_t write_len_t;
     if (buf != command) free(buf);
 
     /* Read server response */
-    resp = (int)( read_server_response_wait (daq->conceal->sockfd, 3) );
+    resp = (int)( read_server_response_wait (daq->conceal->sockfd, 20) );
     if ( nds_logging_check( NDS_LOG_GROUP_VERBOSE_ERRORS,
                             0 ) )
     {
@@ -347,6 +367,7 @@ typedef size_t write_len_t;
         nds_logging_printf( "INFO: %s: exit\n",
                             METHOD );
     }
+    daq->conceal->cur_command_count++;
     return resp;
 }
 
@@ -438,7 +459,8 @@ daq_recv_block (daq_t *daq) {
             if ( nds_logging_check( NDS_LOG_GROUP_VERBOSE_ERRORS,
                                     0 ) )
             {
-                nds_logging_printf ( "ERROR: %s: malloc(%" PRISIZE_T ") failed; errno=%d\n",
+                nds_logging_printf ( "ERROR: %s: malloc(%" PRISIZE_T
+                                     ") failed; errno=%d\n",
                                      METHOD, bsize, errno);
             }
             daq -> tb_size = 0;
@@ -496,7 +518,7 @@ daq_recv_block_num (daq_t *daq) {
     {
         return -1;
     }
-    /*printf("Received block number: %i\n", bnum);*/
+    /* nds_logging_printf("Received block number: %i\n", bnum); */
     return (int)bnum;
 }
 
@@ -1001,6 +1023,25 @@ daq_get_channel_data(daq_t *daq, const char* channel, char* data) {
     return data;
 }
 
+/*  Get the last error message recorded by the server.
+ */
+int
+daq_get_last_message(daq_t *daq, char* buf, size_t max_len, int* len) {
+    int rc = DAQD_OK;
+        
+    switch(daq->nds_versn) {
+    case nds_v1:
+        rc = nds1_get_last_message(daq, buf, max_len, len);
+        break;
+    case nds_v2:
+        rc = nds2_get_last_message(daq, buf, max_len, len);
+        break;
+    default:
+        rc = DAQD_NOT_CONFIGURED;
+    }
+    return rc;
+}
+
 /*  Copy the data from a specified channel name to the target address.
  */
 int
@@ -1221,7 +1262,7 @@ const char* daq_strerror(int errornum)
             retval = "Requested feature is not supported";
             break;
  	case DAQD_ONTAPE:
-            retval = "Data are on tape";
+            retval = "Requested data is on tape";
             break;
         default:
             retval = "unknown error";
@@ -1265,7 +1306,7 @@ daq_startup(void) {
         {
             /* Tell the user that we could not find a usable */
             /* Winsock DLL.                                  */
-            printf("WSAStartup failed with error: %d\n", err);
+            nds_logging_printf("WSAStartup failed with error: %d\n", err);
             return DAQD_ERROR;
         }
         /* Confirm that the WinSock DLL supports MAJOR.MINOR.*/
@@ -1278,7 +1319,7 @@ daq_startup(void) {
         {
             /* Tell the user that we could not find a usable */
             /* WinSock DLL.                                  */
-            printf("Could not find a usable version of Winsock.dll\n");
+            nds_logging_printf( "Could not find a usable version of Winsock.dll\n" );
             WSACleanup();
             return DAQD_ERROR;
         }
@@ -1377,6 +1418,8 @@ cvt_chantype_str(enum chantype t) {
         return "test-pt";
     case cStatic:
         return "static";
+    case cSimData:
+        return "simdata";
     default:
         break;
     }
@@ -1395,6 +1438,7 @@ cvt_str_chantype(const char* name) {
     TEST_NAME(cMTrend);
     TEST_NAME(cTestPoint);
     TEST_NAME(cStatic);
+    TEST_NAME(cSimData);
 #undef TEST_NAME
     return cUnknown;
 }
