@@ -29,7 +29,7 @@
  *    Select includes depend on posix versions
  *--------------------------------------------------------------------*/
 #if _POSIX_C_SOURCE < 200100L
-#if HAVE_SYS_TIME_H
+#if HAVE_SYIME_H
 #include <sys/time.h>
 #endif /* HAVE_SYS_TIME_H */
 #include <sys/types.h>
@@ -88,6 +88,77 @@ nds_str_lastindex(const char *str, int ch)
 	return index;
 }
 
+int
+nds1_reconnect (daq_t *daq)
+{
+    socket_addr_t srv_addr_tmp;
+    size_t max_nds1_commands = 0;
+    int resp = 0;
+    int connect_rc = 0;
+
+    memcpy(&srv_addr_tmp, daq->conceal->srvr_addr, sizeof(srv_addr_tmp));
+    max_nds1_commands = daq->conceal->max_command_count;
+    daq_private_delete( &(daq->conceal) );
+    daq_private_create( &(daq->conceal) );
+
+    /*  Get a socket.
+     */
+    resp = daq_private_srvr_open( daq->conceal );
+    if ( resp != DAQD_OK )
+    {
+        return resp;
+    }
+
+    memcpy(daq->conceal->srvr_addr, &srv_addr_tmp, sizeof(srv_addr_tmp));
+    daq->conceal->max_command_count = max_nds1_commands;
+
+    /*-----------------------------------  Connect to the server      */
+    daq_private_srvr_nonblocking( daq->conceal, 1 );
+    if ( nds_logging_check( NDS_LOG_GROUP_STATUS_UPDATE,
+                            0 ) )
+    {
+        nds_logging_printf("Connecting NDS1 ..");
+        nds_logging_flush( );
+    }
+    connect_rc = daq_private_srvr_connect( daq->conceal );
+    if ( connect_rc == NDS_SOCKET_TRANSIENT_FAILURE )
+    {
+        struct timeval timeout = {4, 0};
+        fd_set writefds;
+        int ret;
+        FD_ZERO(&writefds);
+        FD_SET(daq->conceal->sockfd, &writefds);
+        ret = select(NFDS(daq->conceal->sockfd), NULL, &writefds, NULL, &timeout);
+        if (ret == 1)
+            connect_rc = NDS_SOCKET_OK;
+    }
+    if ( connect_rc != NDS_SOCKET_OK )
+    {
+        if ( nds_logging_check( NDS_LOG_GROUP_STATUS_UPDATE,
+                                0 ) )
+        {
+            char pmsg[ 256 ];
+
+            nds_logging_printf(" failed\n");
+            strerror_r( errno, pmsg, sizeof( pmsg ) );
+            nds_logging_printf( "connect( ): errno: %d - %s\n",
+                                errno, pmsg );
+        }
+        daq_private_srvr_close( daq->conceal );
+        return DAQD_CONNECT;
+    }
+    if ( nds_logging_check( NDS_LOG_GROUP_STATUS_UPDATE,
+                            0 ) )
+    {
+        nds_logging_printf(" done\n");
+    }
+    daq_private_srvr_nonblocking( daq->conceal, 0 );
+
+    daq -> conceal -> datafd = daq -> conceal -> sockfd;
+
+    return 0;
+}
+
 /*
  *  Connect to the DAQD server on the host identified by `ip' address.
  *  Returns zero if OK or the error code if failed.
@@ -281,6 +352,16 @@ nds1_disconnect (daq_t *daq) {
     return 0;
 }
 
+/*
+ *  Get the most recent error message recorded by the server.
+ *  Returns zero if OK or the error code if failed.
+ */
+int
+nds1_get_last_message (daq_t *daq, char* buf, size_t max_len, int* str_len) {
+    int len = strerror_r(daq->err_num, buf, max_len);
+    if (str_len) *str_len = len;
+    return len ? DAQD_OK : DAQD_ERROR;
+}
 
 static void
 cvthex(const char** s, char* buf, void* x) {
@@ -351,7 +432,7 @@ _nds1_read_sc3_number(nds_socket_type sock, size_t max_len, char *buf, long *out
     tmp = strtol(buf, &endp, 10);
     if (*endp != '\0')
     {
-        perror("Error channel meta-data");
+        nds_logging_print_errno( "Error channel meta-data" );
         return DAQD_ERROR;   
     }
     *out = tmp;
@@ -371,7 +452,7 @@ _nds1_read_sc3_double(nds_socket_type sock, size_t max_len, char *buf, double *o
     tmp = strtod(buf, &endp);
     if (*endp != '\0')
     {
-        perror("Error channel meta-data");
+        nds_logging_print_errno( "Error channel meta-data: %s\n" );
         return DAQD_ERROR;   
     }
     *out = tmp;
@@ -431,7 +512,7 @@ typedef size_t write_len_t;
      */
     if ( write (daq->conceal->sockfd, chan_req, (write_len_t) strlen(chan_req)) != strlen(chan_req) )
     {
-        perror("Error in write()");
+        nds_logging_print_errno( "Error in write(): %s" );
         return DAQD_WRITE;
     }
     /* On success the output from the server is the channel name, not 0000.
@@ -440,13 +521,13 @@ typedef size_t write_len_t;
      */
     if ( read_bytes( daq->conceal->sockfd, buf, (size_t)1 ) != 1 ) 
     {
-        perror("Error in read_bytes()");
+        nds_logging_print_errno( "Error in read_bytes()" );
         return DAQD_ERROR;
     }
     /* Error condition */
     if ( buf[0] == '0' ) {
         if ( read_bytes( daq->conceal->sockfd, buf + 1, (size_t)3 ) != 3 ) {
-            perror("Error in read_bytes()");
+            nds_logging_print_errno( "Error in read_bytes()" );
             return DAQD_ERROR;
         }
         buf[4] = '\0';
@@ -509,8 +590,9 @@ typedef size_t write_len_t;
     *num_channels_received = 1;
 clean_up:
     if (result != DAQD_OK)
-        perror("Unable to read channel meta-data");
-
+    {
+        nds_logging_print_errno( "Unable to read channel meta-datan" );
+    }
     return result;
 #if defined(BUF_SIZE)
 #undef BUF_SIZE
@@ -642,7 +724,7 @@ nds1_recv_channels_impl (daq_t *daq, daq_channel_t *channel, int num_channels,
     for (i = 0; i < channels; i++) {
 #if _WIN32
         if (_daq_read_cstring(f, chan_buf_size, buf ) <= 0) {
-            perror("string read failed");
+            nds_logging_print_errno( "string read failed" );
             return DAQD_ERROR;
         }
         if ( nds_logging_check( NDS_LOG_GROUP_VERBOSE_ERRORS,
@@ -654,12 +736,12 @@ nds1_recv_channels_impl (daq_t *daq, daq_channel_t *channel, int num_channels,
 
 #else /* _WIN32 */
         if (fgets(buf, (int)( chan_buf_size ), f) <= 0) {
-            perror("fgets failed");
+            nds_logging_print_errno( "fgets failed" );
             fclose(f);
             return DAQD_ERROR;
         }
 #endif /* _WIN32 */
-        /* printf("chan string: %s\n", buf); */
+        /* nds_logging_printf("chan string: %s", buf); */
 
         if (i < num_channels) {
             int intval;
