@@ -105,7 +105,7 @@ nds2_connect (daq_t *daq, const char* host, int port)
     /*----------------------------------  Get the server address.     */
     resp = daq_set_server(daq, host, port);
     if ( resp != DAQD_OK ) {
-        perror("Error in daq_set_server");
+        nds_logging_print_errno( "Error in daq_set_server" );
         daq_private_srvr_close( daq->conceal );
         if ( nds_logging_check( NDS_LOG_GROUP_TRACE_ENTRY_EXIT, 10 ) )
         {
@@ -293,6 +293,48 @@ nds2_disconnect (daq_t *daq) {
     return 0;
 }
 
+/*
+ *  Get the most recent error message recorded by the server.
+ *  Returns zero if OK or the error code if failed.
+ */
+int
+nds2_get_last_message (daq_t *daq, char* message, size_t maxlen, int* str_len) {
+    const char* METHOD = "nds2_get_last_message";
+    const char* cmd = "get-last-message;";
+    int resp, tmp_len;
+    nds_socket_type fd;
+
+    if (str_len) *str_len = 0;
+    if (maxlen == 0) return DAQD_ERROR;
+
+    if ( nds_logging_check( NDS_LOG_GROUP_VERBOSE_ERRORS, 10 ) ) {
+        nds_logging_printf( "INFO: %s: Requesting last message\n", METHOD );
+    }
+
+    if ( nds_logging_check( NDS_LOG_GROUP_VERBOSE_ERRORS, 10 ) )
+    {
+        nds_logging_printf("INFO: %s: buf - %s\n",
+                           METHOD,  cmd );
+    }
+    resp = daq_send(daq, cmd);
+    if (resp) {
+        if ( nds_logging_check( NDS_LOG_GROUP_VERBOSE_ERRORS, 0 ) )
+        {
+            nds_logging_printf("ERROR: %s: Error return from daq_send(): %s\n",
+                               METHOD,
+                               daq_strerror(resp));
+        }
+        return resp;
+    }
+
+    /*----------------------------------  Read the source string      */
+    fd = daq->conceal->sockfd;
+    tmp_len = _daq_read_string(fd, maxlen, message);
+    if (tmp_len < 0) return DAQD_ERROR;
+    *str_len = tmp_len;
+    return resp;
+}
+
 #define BUF_LENGTH 512UL
 #define MAX_TYPESTR_LENGTH 16
 
@@ -718,7 +760,7 @@ nds2_recv_source_data(daq_t *daq, char* list, size_t max_len, time_t gps,
     cmd_txt = 0;
 
     if (rc) {
-        printf("Error in daq_send: %s\n", daq_strerror(rc));
+        nds_logging_printf("Error in daq_send: %s\n", daq_strerror(rc));
         return rc;
     }
 
@@ -770,7 +812,7 @@ nds2_recv_source_list(daq_t *daq, char* list, size_t max_len, time_t gps,
     cmd_txt = 0;
 
     if (rc) {
-        printf("Error in daq_send: %s\n", daq_strerror(rc));
+        nds_logging_printf("Error in daq_send: %s\n", daq_strerror(rc));
         return rc;
     }
 
@@ -780,6 +822,64 @@ nds2_recv_source_list(daq_t *daq, char* list, size_t max_len, time_t gps,
     return rc;
 }
 
+/*  nds2_request_data(daq_t*, time_t, time_t)
+ *
+ *  Get requested channel data for the specified interval.
+ */
+int
+nds2_request_check(daq_t* daq, time_t start, time_t end) {
+    static const char* METHOD = "nds2_request_check";
+    int    rc = DAQD_OK;
+    size_t N, namel, txt_len;
+    char   *cmd, *cmd_txt, *p;
+
+    /*  Get the request type and sum channel name length
+     */
+    N = daq->num_chan_request;
+    if (N == 0) {
+        if ( nds_logging_check( NDS_LOG_GROUP_VERBOSE_ERRORS,
+                                0 ) )
+        {
+            nds_logging_printf( "ERROR: %s: No channels requested\n",
+                                METHOD );
+        }
+        return DAQD_ERROR;
+    }
+
+    namel = _nds2_name_list_length(daq);
+
+    /*----------------------------------  Build up a command
+     */
+    cmd = "check-data";
+
+    /*----------------------------------  Get the text length, allocate a buffer
+     */
+    txt_len = strlen(cmd)
+            + strlen(" xxxxxxxxxx xxxxxxxxxx xxxxxxxxxx {};") 
+            + namel + N;
+    cmd_txt = (char*)malloc(txt_len);
+
+    /*----------------------------------  Build the command
+     */
+    p = cmd_txt;
+    while (*cmd) *p++ = *cmd++;
+    *p++ = ' ';
+    p += daq_cvt_timet_string(start, p);
+    *p++ = ' ';
+    p += daq_cvt_timet_string(end, p);
+    *p++ = ' ';
+    p += _nds2_get_name_list(daq, p);
+    *p++ = ';';
+    *p++ = 0;  
+
+    /*----------------------------------  Send the command free the command
+     */
+    rc = daq_send(daq, cmd_txt);
+    free(cmd_txt);
+    cmd_txt = 0;
+    return rc;
+}
+
 /*  nds2_request_data(daq_t*, time_t, time_t)
  *
  *  Get requested channel data for the specified interval.
@@ -859,7 +959,7 @@ nds2_request_data(daq_t* daq, time_t start, time_t end, time_t dt) {
 
 
     if (rc) {
-        printf("Error in daq_send: %s\n", daq_strerror(rc));
+        nds_logging_printf("Error in daq_send: %s\n", daq_strerror(rc));
         return rc;
     }
 
@@ -867,7 +967,7 @@ nds2_request_data(daq_t* daq, time_t start, time_t end, time_t dt) {
      */
     wid = (int)( daq_recv_id(daq) );
     if (wid < 0) {
-        printf("Error reading writerID in daq_recv_block\n");
+        nds_logging_printf("Error reading writerID in daq_recv_block\n");
         return DAQD_ERROR;
     }
 
@@ -875,13 +975,12 @@ nds2_request_data(daq_t* daq, time_t start, time_t end, time_t dt) {
      */
     bl = daq_recv_block_num(daq);
     if (bl < 0) {
-        printf("Error reading offline flag in daq_recv_block\n");
+        nds_logging_printf("Error reading offline flag in daq_recv_block\n");
         rc = DAQD_ERROR;
     }
     return rc;
 }
 
-
 struct nds2_chan_status {
     int status;
     int offset;
@@ -896,38 +995,65 @@ struct nds2_chan_status {
  *  Receive a reconfigure block. receive_reconfigure is received after the 
  *  block header has been read in. The block length does not include the
  *  header length.
+ *
+ *  Valid return codes are:
+ *  -1  Error Attempting to read a reconfigure block.
+ *  -2  Successfully read reconfigure block.
  */
 int 
 nds2_receive_reconfigure(daq_t* daq, size_t block_len) {
     nds_socket_type fd;
     size_t nchannels, i;
+    size_t status_len, byte_count;
     chan_req_t* list;
-
-    nchannels = block_len / sizeof(struct nds2_chan_status);
-    if (nchannels * sizeof(struct nds2_chan_status) != block_len) {
-        if ( nds_logging_check( NDS_LOG_GROUP_STATUS_UPDATE,
-                                0 ) )
-        {
-            nds_logging_printf( "Channel reconfigure block length has bad length (%" PRISIZE_T ")\n",
-                                block_len );
+    status_len=sizeof(struct nds2_chan_status);
+    nchannels = daq->num_chan_request;
+    if (daq->nds1_rev <= 5) {
+        if (nchannels * status_len != block_len) {
+            if ( nds_logging_check( NDS_LOG_GROUP_STATUS_UPDATE,
+                                    0 ) )
+                {
+                    nds_logging_printf( "Channel reconfigure block length has "
+                                        "bad length (%" PRISIZE_T ")\n",
+                                        block_len );
+                }
+            return -1;
         }
     }
-    if (nchannels != daq->num_chan_request) return -1;
-
+    
     fd = daq->conceal->datafd;
     list = daq->chan_req_list;
-    for (i = 0; i < nchannels; i++) {
+    byte_count = 0;
+    for (i = 0; i < nchannels && byte_count < block_len; i++) {
         float rate;
         uint4_type dtype;
-        if (read_uint4(fd, (uint4_type*)&list[i].status))  return DAQD_ERROR;
-        if (read_uint4(fd, &list[i].offset))  return DAQD_ERROR;
-        if (read_uint4(fd, &dtype))           return DAQD_ERROR;
+        if (read_uint4(fd, (uint4_type*)&list[i].status))  return -1;
+        if (read_uint4(fd, &list[i].offset))  return -1;
+        if (read_uint4(fd, &dtype))           return -1;
         if (list[i].type == cUnknown) list[i].type = dtype >> 16;
         list[i].data_type = (dtype & 0xffff);
-        if (read_float(fd, &rate))           return DAQD_ERROR;
+        if (read_float(fd, &rate))           return -1;
         list[i].rate = rate;
-        if (read_float(fd, &list[i].s.signal_offset))  return DAQD_ERROR;
-        if (read_float(fd, &list[i].s.signal_slope))   return DAQD_ERROR;
+        if (read_float(fd, &list[i].s.signal_offset))  return -1;
+        if (read_float(fd, &list[i].s.signal_slope))   return -1;
+        byte_count += status_len;
+        if (daq->nds1_rev > 5) {
+            int str_len;
+            str_len = _daq_read_string(fd, MAX_SIGNAL_UNIT_LENGTH,
+                                       list[i].s.signal_units);
+            if (str_len < 0) return -1;
+            byte_count += sizeof(uint4_type) + (unsigned) str_len;
+        }
+    }
+    if (byte_count != block_len) {
+        if ( nds_logging_check( NDS_LOG_GROUP_STATUS_UPDATE, 0 ) )
+            {
+                nds_logging_printf( "Channel reconfigure block length has "
+                                    "bad length (%" PRISIZE_T ") expected (%"
+                                    PRISIZE_T "\n",
+                                    byte_count, block_len);
+            }
+        return -1;
     }
     return -2; 
 }
@@ -959,9 +1085,11 @@ static int
 get_version_and_revision( daq_t* daq )
 {
     int retval;
+    char cmd_buf[32];
     uint4_type v = UNSPECIFIED_NDS2_PROTOCOL_VERSION;
     uint4_type rv = UNSPECIFIED_NDS2_PROTOCOL_REVISION;
     uint4_type exp_rv = UNSPECIFIED_NDS2_PROTOCOL_REVISION;
+    uint4_type max_rv = MAX_NDS2_PROTOCOL_REVISION;
 
     /*-------------------------------------------------------------------
      * Obtain the server protocol version
@@ -1008,17 +1136,31 @@ get_version_and_revision( daq_t* daq )
                                 0 ) )
         {
             nds_logging_printf( "unsupported communication protocol version: ");
-            nds_logging_printf( "received %d, only support %d to %d\n",v,MIN_NDS2_PROTOCOL_VERSION, MAX_NDS2_PROTOCOL_VERSION);
-            nds_logging_printf(" version %d not between %d to %d\n",v,MIN_NDS2_PROTOCOL_VERSION,MAX_NDS2_PROTOCOL_VERSION);
+            nds_logging_printf( "received %d, only support %d to %d\n", v,
+                                MIN_NDS2_PROTOCOL_VERSION,
+                                MAX_NDS2_PROTOCOL_VERSION);
+            nds_logging_printf(" version %d not between %d to %d\n", v,
+                               MIN_NDS2_PROTOCOL_VERSION,
+                               MAX_NDS2_PROTOCOL_VERSION);
         }
         retval = DAQD_VERSION_MISMATCH;
         goto error_recovery;
     }
     daq->nds1_ver = v;        /* server protocol version */
     /*-------------------------------------------------------------------
-     * Obtain the server protocol revision
+     * Obtain the server protocol revision (supported in revision 1.6+)
      *-------------------------------------------------------------------*/
-    retval = daq_send (daq, "server-protocol-revision;");
+    sprintf(cmd_buf, "server-protocol-revision %d;",
+            MAX_NDS2_PROTOCOL_REVISION);
+    retval = daq_send (daq, cmd_buf);
+    /*-------------------------------------------------------------------
+     * Handle negotiation with an older server cleanly
+     *-------------------------------------------------------------------*/
+    if (retval == DAQD_COMMAND_SYNTAX)
+    {
+        sprintf(cmd_buf, "server-protocol-revision;");
+        retval = daq_send (daq, cmd_buf);
+    }
     switch( retval )
     {
     case DAQD_OK:
@@ -1056,7 +1198,7 @@ get_version_and_revision( daq_t* daq )
         exp_rv = NDS2_VERSION_1_REVISION;
     }
     /* Determine if the expected revision was received */
-    if (rv != exp_rv) {
+    if (rv < exp_rv || rv > max_rv) {
         if ( nds_logging_check( NDS_LOG_GROUP_STATUS_UPDATE,
                                 0 ) )
         {
