@@ -1,9 +1,9 @@
 #!/bin/sh
 # Run an m-file inside MATLAB from the command line.
-matlab -nodisplay -r "\
+${MATLAB} -nodisplay -r "\
   javaaddpath('${builddir}'); \
   try; \
-    run('$(python -c "import os; print(os.path.abspath('$1'))")'); \
+    run('${abs_srdir}/${1}'); \
     ret = 0; \
   catch ex; \
     ret = 1; \
