@@ -2,12 +2,8 @@ import nds2
 import os
 import sys
 
-host = 'localhost'
-if os.environ.has_key('NDS_TEST_HOST'):
-    host = os.environ['NDS_TEST_HOST']
-port = 31200
-if os.environ.has_key('NDS_TEST_PORT'):
-    port = int(os.environ['NDS_TEST_PORT'])
+host = os.getenv('NDS_TEST_HOST', 'localhost')
+port = int(os.getenv('NDS_TEST_PORT', 31200))
 
 c = nds2.connection(host, port, nds2.connection.PROTOCOL_ONE)
 
